/*************************************************************************
 *
 *    Used with ICCARM and AARM.
 *
 *    (c) Copyright IAR Systems 2005
 *
 *    File name   : main.c
 *    Description : Define main module
 *
 *    History :
 *    1. Data        : June 24, 2005
 *       Author      : Stanimir Bonev
 *       Description : Create
 *    2. Data        : Nov 1, 2005
 *       Author      : Stanimir Bonev
 *       Description : modify (new revision of the board)
 *
 * Jumpers:
 *  EXT/JLINK - depend of power source
 *  M0.L/M0.H - 2-3
 *  M1.L/M1.H - 2-3
 *  R-T       - Absence
 *
 * Terminal setings
 *  Baud rate = 9600
 *  Data bits = 8
 *  Parity    = no
 *  Stop bits = 1
 *  Flow control = no
 *
 *    $Revision: 1.1 $
 *
**************************************************************************/

#include "includes.h"

/// LCD REVISION



#define BUF ((struct uip_eth_hdr *)&uip_buf[0])

typedef enum {
 TEST_BEGIN_S = 0, SKIP_EXT_RESPOND_WAIT,
 EXT_TEST_BEGIN_S, EXT_TEST_PASS_2_S, EXT_TEST_PASS_3_S, EXT_TEST_PASS_4_S,
 CAN_TEST_S, UEXT_TEST_S,
 ADC_TEST_S, BATTERY_TEST_S, BATTERY_TEST_INIT_S,
 FUNC_TEST_BEGIN_S,
 FUNC_LAN_TEST_S, FUNC_USB_TEST_S,FUNC_AUDIO_TEST_S,FUNC_AUDIO_TEST_RESPOND_S,
 FUNC_RTC_TEST_S,
 FUNC_MMC_TEST_S, FUNC_MMC_PRESEN_TEST_S, FUNC_MMC_PROT_TEST_S,
 FUNC_IR_TEST_S, FUNC_IR_TEST_BEGIN_S,FUNC_IR_TEST_CONT_S,
 FUNC_JOYSTICK_TEST_S,FUNC_JOYSTICK_INIT_S,

} test_state_t;

#define CAN1_RX   27
#define CAN1_TX   26

#define UEXT_PIN_3    24
#define UEXT_PIN_4    25
#define UEXT_PIN_5    16
#define UEXT_PIN_6    17
#define UEXT_PIN_7    22
#define UEXT_PIN_8    21
#define UEXT_PIN_PULL 20
//#define UEXT_PIN_10   19
#define UEXT_PIN_10   23

#define BUT_CENTER    44
#define LCD_LIGHT     52


const char TetsBegin[]      = "\r\nTets begin\r\n";
const char NewLine[]        = "\r\n";
const char SkipExtTest []   = "Skip extension test (y/n):";
const char ExtTest[]        = "Extension test";
const char ExtTestFault[]   = "\r\nExtension test fault !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\r\n";
const char TestPass[]       = "- pass\r\n";
const char TestFault[]      = "- fault !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\r\n";
const char FuncTest[]       = "Function test begin\r\n";
const char UextTestMsg[]    = "\n\rPlease remove EXT and EMI extension, put UEXT and press any key.";
const char ADCTestMsg[]    = "ANALOG test - press any key.";

const char TestSkipped[]    = "- skipped !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\r\n";

const char LanTest[]        = "LAN test (go to 192.168.0.100) or press eny key to skip test";
const char USBTest[]        = "USB test (Connect the USB cable) or press eny key to skip test";
const char AudioTest[]      = "Audio test pass (y/n):";
const char RTC_test[]       = "RTC Test ";
const char JoystickTest[]   = "\r\nNow test Joystick and AN_TR\r\n";
const char TestMMC[]        = "MMC IO test. Please insert the MMC card or press any key to skip the test";
const char TestWP_MMC[]     = "MMC write protect test. Take the card change the WP switch position and put it again or press any key to skip the test";
const char StallProtected[] = "\r\nStall on protected state !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\r\n";
const char StallUnProtected[]= "\r\nStall on unprotected state !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\r\n";

const char TestIR[]         = "IR Test. Move jumper CAN_RX/U1_TX on U1_TX position, CAN_TX/U1_RX on U1_RX position and press (y) test start or (SPACE_KEY) for test skip: ";

const char FuncTestFault[]  = "\r\nFunction test fault !!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!!\r\n";

const char CommPullErr[]    = "\r\nNot connection between common pull %d pin and %d pin\r\n";
const char NotConnect[]     = "\r\nThe connection to pin %d is cut off\r\n";
const char ShortErr[]       = "\r\nShort between %d pin and %d\r\n";
const char UartErr[]        = "\r\nShort between Uart %d and Uart %d\r\n";
const char UartNoise[]      = "\r\nUart %d receive noise\r\n";
const char UartNoConn[]     = "\r\nNot connection between Uart %d and Uart %d\r\n";
const char CanErr[]         = "\r\nCAN test failt\r\n";
const char CanReport[]      = "\r\nPlease open CAN_E djumper and press anu key";
const char AdcTestFault[]   = "\r\nADC channel %d test fault";
const char I2C1Err[]        = "\r\nI2C%d test failt\r\n";
const char MMCAgain[]       = "\r\nPlease move CARD_CS_SEL at second position. Press any key...";



const char TermErr[] = " Term-Er";
const char TermOK[] = " Term-OK";

const char TrimErr[] = "Trim-Er";
const char TrimOK[] = "Trim-OK";


const char ButtonsStr[] = "Buttons:     ";
char Line1Str[17];
char Line2Str[17];

const Int8U HexToCharStr [] = "0123456789ABCDEF";

Boolean LedTestEna;

test_state_t test_state = TEST_BEGIN_S;

volatile unsigned int ticks = 0;
unsigned int Tim0Per;
char about_fl;
char flag;
char trim_flag;

struct
{
  Int8U PatternState;
  Int8U Pattern[16];
}LedPattern = {15,0,1,2,3,4,5,6,7,8,9,10,11,12,13,14,15};

void Delay (Int32U Dly)
{
volatile Int32U Dly1 = Dly;
  for(;Dly1;--Dly1);
}

void Dly100us (void *arg)
{
volatile Int32U Dly = (Int32U)arg;
  for(;Dly;--Dly)
  {
    Delay(300);
  }
}

Boolean CanTest (void)
{
Boolean Error = 0;
// Test CAN1
  SetDir(CAN1_TX,1);
  SetDir(CAN1_RX,0);
  SetOut(CAN1_TX,1);
  Delay(100);
  if(GetIn(CAN1_RX) != 1)
  {
    printf(CanErr);
    Error = 1;
  }

  SetOut(CAN1_TX,0);
  Delay(100);
  if(GetIn(CAN1_RX) != 0)
  {
    printf(CanErr);
    Error = 1;
  }

  printf(CanReport);
  getchar();

  SetOut(CAN1_TX,1);
  Delay(100);
  if(GetIn(CAN1_RX) != 1)
  {
    printf(CanErr);
    Error = 1;
  }

  SetOut(CAN1_TX,0);
  Delay(100);
  if(GetIn(CAN1_RX) != 1)
  {
    printf(CanErr);
    Error = 1;
  }

  return Error;
}

Boolean UextTest (void)
{
// Test UEXT
Boolean Error = 0;
    // Setting
    SetDir(UEXT_PIN_PULL,1);
    SetDir(UEXT_PIN_3,0); SetDir(UEXT_PIN_4,0); SetDir(UEXT_PIN_5,0);
    SetDir(UEXT_PIN_6,0); SetDir(UEXT_PIN_7,0); SetDir(UEXT_PIN_8,0);
    SetDir(UEXT_PIN_10,0);

    // pull up low -> all low
    SetOut(UEXT_PIN_PULL,0);
    Delay(100);
    if( (GetIn(UEXT_PIN_3)!=0)||(GetIn(UEXT_PIN_4)!=0)||(GetIn(UEXT_PIN_5)!=0)||(GetIn(UEXT_PIN_6)!=0)||
       (GetIn(UEXT_PIN_7)!=0)||(GetIn(UEXT_PIN_8)!=0)||(GetIn(UEXT_PIN_10)!=0) ) {

        printf(ExtTestFault);
        test_state = TEST_BEGIN_S;
        Error = 1;
    }

    // Pull up high -> all high
    SetOut(UEXT_PIN_PULL,1);
    Delay(100);
    if( (GetIn(UEXT_PIN_3)!=1)||(GetIn(UEXT_PIN_4)!=1)||(GetIn(UEXT_PIN_5)!=1)||(GetIn(UEXT_PIN_6)!=1)||
        (GetIn(UEXT_PIN_7)!=1)||(GetIn(UEXT_PIN_8)!=1)||(GetIn(UEXT_PIN_10)!=1) ) {

        printf(ExtTestFault);
        test_state = TEST_BEGIN_S;
        Error = 1;
    }

    // test short of PIN3 and other
    SetDir(UEXT_PIN_3,1);
    SetOut(UEXT_PIN_3,0);
    Delay(100);
    if( (GetIn(UEXT_PIN_4)!=1)||(GetIn(UEXT_PIN_5)!=1)||(GetIn(UEXT_PIN_6)!=1)||
        (GetIn(UEXT_PIN_7)!=1)||(GetIn(UEXT_PIN_8)!=1)||(GetIn(UEXT_PIN_10)!=1) ) {

        printf(ExtTestFault);
        test_state = TEST_BEGIN_S;
        Error = 1;
    }
    SetDir(UEXT_PIN_3,0);

    // test short of PIN4 and other
    SetDir(UEXT_PIN_4,1);
    SetOut(UEXT_PIN_4,0);
    Delay(100);
    if( (GetIn(UEXT_PIN_3)!=1)||(GetIn(UEXT_PIN_5)!=1)||(GetIn(UEXT_PIN_6)!=1)||
        (GetIn(UEXT_PIN_7)!=1)||(GetIn(UEXT_PIN_8)!=1)||(GetIn(UEXT_PIN_10)!=1) ) {

        printf(ExtTestFault);
        test_state = TEST_BEGIN_S;
        Error = 1;
    }
    SetDir(UEXT_PIN_4,0);

    // test short of PIN10 and other
    SetDir(UEXT_PIN_10,1);
    SetOut(UEXT_PIN_10,0);
    Delay(100);
    if( (GetIn(UEXT_PIN_4)!=1)||(GetIn(UEXT_PIN_5)!=1)||(GetIn(UEXT_PIN_6)!=1)||
        (GetIn(UEXT_PIN_7)!=1)||(GetIn(UEXT_PIN_8)!=1)||(GetIn(UEXT_PIN_3)!=1) ) {

        printf(ExtTestFault);
        test_state = TEST_BEGIN_S;
        Error = 1;
    }
    SetDir(UEXT_PIN_10,0);


    return Error;
}



// UART0 Send char
void Uart0SendChar (Int8U c)
{
  UART_SendData(UART0, c);
  while(UART_GetFlagStatus(UART0, UART_FLAG_TxFIFOFull) != RESET);
}

// UART1 Send char
void Uart1SendChar (Int8U c)
{
  UART_SendData(UART1, c);
  while(UART_GetFlagStatus(UART1, UART_FLAG_TxFIFOFull) != RESET);
}

// UART2 Send char
void Uart2SendChar (Int8U c)
{
  UART_SendData(UART2, c);
  while(UART_GetFlagStatus(UART2, UART_FLAG_TxFIFOFull) != RESET);
}

// UART0 receive char
Int8U Uart0ReceiveChar (void)
{
Int8U Data;
  while(UART_GetFlagStatus(UART0, UART_FLAG_RxFIFOEmpty) == SET);
  Data = UART0->DR;
  return(Data);
}

// UART1 receive char
Int8U Uart1ReceiveChar (void)
{
Int8U Data;
  while(UART_GetFlagStatus(UART1, UART_FLAG_RxFIFOEmpty) == SET);
  Data = UART1->DR;
  return(Data);
}

// UART2 receive char
Int8U Uart2ReceiveChar (void)
{
Int8U Data;
  while(UART_GetFlagStatus(UART2, UART_FLAG_RxFIFOEmpty) == SET);
  Data = UART2->DR;
  return(Data);
}

// UART0 receive char test
Int8U Uart0ReceiveCharTest (void)
{
  return(UART_GetFlagStatus(UART0, UART_FLAG_RxFIFOEmpty) != SET);
}

// UART1 receive char
Int8U Uart1ReceiveCharTest (void)
{
  return(UART_GetFlagStatus(UART1, UART_FLAG_RxFIFOEmpty) != SET);
}

// UART2 receive char
Int8U Uart2ReceiveCharTest (void)
{
  return(UART_GetFlagStatus(UART2, UART_FLAG_RxFIFOEmpty) != SET);
}

// Get ch
int getchar(void)
{
  return(Uart0ReceiveChar());
}

// Send char
int putchar (int c)
{
  Uart0SendChar(c);
  return(c);
}
// kbdhit
Boolean kbdhit(void)
{
  return(Uart0ReceiveCharTest());
}

/*************************************************************************
 * Function Name: TransmitPacket
 * Parameters: none
 *
 * Return: none
 *
 * Description: Transmit data
 *
 *************************************************************************/
void TransmitPacket(void)
{
int i;
Int8U data[1500];

  // Copy the header portion part
  for(i=0; i < (UIP_LLH_LEN + 40); ++i) {
   data[i] =  uip_buf[i];
  }

  // Copy the data portion part
  for(; i < uip_len; ++i) {
    data[i] =  uip_appdata[i - UIP_LLH_LEN - 40 ];
  }

  tapdev_send(&data,uip_len);
}

/*************************************************************************
 * Function Name: uip_log
 * Parameters: none
 *
 * Return: none
 *
 * Description: Events loggin
 *
 *************************************************************************/
void uip_log (char *m)
{
  printf("uIP log message: %s\n", m);
}

/*************************************************************************
 * Function Name: main
 * Parameters: none
 *
 * Return: none
 *
 * Description: main
 *
 *************************************************************************/
int main()
{
Int32U i,j,a;
Boolean Error;
char ch;
Int8U Message[17];
Int32U Tmp, Tmp1, ProtStateCntr;
Boolean WriteProtect,WriteProtectTmp;

UART_InitTypeDef UART_InitStructure;
GPIO_InitTypeDef GPIO_InitStructure;
unsigned int arptimer;
pDiskStatus_t pMMCDiskStatus;

  SCU_MCLKSourceConfig(SCU_MCLK_OSC);
  SCU_FMICLKDivisorConfig(SCU_FMICLK_Div2);
  FMI_Config(FMI_READ_WAIT_STATE_2,FMI_WRITE_WAIT_STATE_0, FMI_PWD_ENABLE,\
             FMI_LVD_ENABLE,FMI_FREQ_HIGH);

  SCU_HCLKDivisorConfig(SCU_HCLK_Div1);
  SCU_PCLKDivisorConfig(SCU_PCLK_Div2);
  SCU_PLLFactorsConfig(N,M,P);              /* PLL = 96 MHz */
  SCU_PLLCmd(ENABLE);                       /* PLL Enabled  */
  SCU_MCLKSourceConfig(SCU_MCLK_PLL);       /* MCLK = PLL   */

  SCU->EMPTY2[0] = 0;
  SCU->EMPTY2[1] = 0;

  LedTestEna = FALSE;

  SCU_APBPeriphClockConfig(__GPIO0, ENABLE);
  GPIO_DeInit(GPIO0);
  SCU_APBPeriphClockConfig(__GPIO1, ENABLE);
  GPIO_DeInit(GPIO1);
  SCU_APBPeriphClockConfig(__GPIO2, ENABLE);
  GPIO_DeInit(GPIO2);
  SCU_APBPeriphClockConfig(__GPIO3, ENABLE);
  GPIO_DeInit(GPIO3);
  SCU_APBPeriphClockConfig(__GPIO4, ENABLE);
  GPIO_DeInit(GPIO4);
  SCU_APBPeriphClockConfig(__GPIO5, ENABLE);
  GPIO_DeInit(GPIO5);
  SCU_APBPeriphClockConfig(__GPIO6, ENABLE);
  GPIO_DeInit(GPIO6);
  SCU_APBPeriphClockConfig(__GPIO7, ENABLE);
  GPIO_DeInit(GPIO7);
  SCU_APBPeriphClockConfig(__GPIO8, ENABLE);
  GPIO_DeInit(GPIO8);
  SCU_APBPeriphClockConfig(__GPIO9, ENABLE);
  GPIO_DeInit(GPIO9);

  SCU_APBPeriphClockConfig(__UART0, ENABLE);
  UART_DeInit(UART0);
  SCU_APBPeriphClockConfig(__UART1, ENABLE);
  UART_DeInit(UART1);
  SCU_APBPeriphClockConfig(__UART2, ENABLE);
  UART_DeInit(UART2);

  UART_Cmd(UART0, DISABLE);
  UART_Cmd(UART1, DISABLE);
  UART_Cmd(UART2, DISABLE);

  /* Gonfigure P4.0*/
  GPIO_ANAPinConfig(GPIO_ANAChannelALL,ENABLE);

  /*Gonfigure UART1_Tx pin GPIO3.3*/
  GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
  GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull ;
  GPIO_InitStructure.GPIO_IPConnected = GPIO_IPConnected_Disable;
  GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt1  ;
  GPIO_Init (GPIO3, &GPIO_InitStructure);
  GPIO_WriteBit(GPIO3,GPIO_Pin_3,Bit_RESET);

  /*Gonfigure UART0_Rx pin GPIO5.1*/
  GPIO_InitStructure.GPIO_Direction = GPIO_PinInput;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_1;
  GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull ;
  GPIO_InitStructure.GPIO_IPConnected = GPIO_IPConnected_Enable;
  GPIO_InitStructure.GPIO_Alternate = GPIO_InputAlt1  ;
  GPIO_Init (GPIO5, &GPIO_InitStructure);

  /*Gonfigure UART0_Tx pin GPIO5.0*/
  GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
  GPIO_InitStructure.GPIO_Pin = GPIO_Pin_0;
  GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull ;
  GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt3  ;
  GPIO_Init (GPIO5, &GPIO_InitStructure);

  UART_StructInit(&UART_InitStructure);
  UART_InitStructure.UART_WordLength = UART_WordLength_8D;
  UART_InitStructure.UART_StopBits = UART_StopBits_1;
  UART_InitStructure.UART_Parity = UART_Parity_No ;
  UART_InitStructure.UART_BaudRate = 115200;
  UART_InitStructure.UART_HardwareFlowControl = UART_HardwareFlowControl_None;
  UART_InitStructure.UART_Mode = UART_Mode_Tx_Rx;
  UART_InitStructure.UART_FIFO = UART_FIFO_Disable;

  UART_Init(UART0, &UART_InitStructure);

  /* Enable the UART0 */
  UART_Cmd(UART0, ENABLE);


  /* Init and print LCD */
  InitLCD();
  LCDSettings();
  LCDWrite130x130bmp();

  printf(TetsBegin);
  printf(NewLine);

  SCU_AHBPeriphClockConfig(__VIC, ENABLE);
  VIC_DeInit();

  // Init Timer0
  SCU_PRR1_bit.RST_TIM01 = 0;       // Reset of TIM01
  SCU_PCGR1_bit.TIM01 = 1;          // Enable clock of TIM01
  SCU_PRR1_bit.RST_TIM01 = 1;       // Release reset of TIM01
  TIM0_CR2_bit.CC = TIM_PSC(TIM_0_RES)-1; // Set TIM01 perscaler
  TIM0_CR2_bit.OC1IE = 1;           // TIM0 output compare interrupt enable
  // Set timer period
  Tim0Per   = TIM0_PER(TICK_PER_SEC);
  TIM0_OC1R = TIM0_CNTR + Tim0Per;

  /* VIC Configuration */
  VIC_Config(TIM0_ITLine,VIC_IRQ , VIC_Slot3);
  VIC_ITCmd(TIM0_ITLine, ENABLE);
  TIM0_CR1_bit.EN = 1;              // enable counting

  // Initialize the ethernet device driver
  tapdev_init();

  // Init USB Hw
  USB_HwInit(VIC_Slot0,VIC_Slot1);
  // Init USB Core
  UsbCoreInit(UsbClassAudioConfigure);
  // Init Audio
  AudioClassInit(VIC_Slot2);

  // Enable interrupts
  __enable_interrupt();

  // uIP web server

  // Initialize the uIP TCP/IP stack.
  uip_init();

  // Initialize the HTTP server.
  httpd_init();

  arptimer = 0;


  while(1)
  {
    // Test Statets
    switch(test_state)
    {
    case TEST_BEGIN_S:
      printf(SkipExtTest);
      test_state = SKIP_EXT_RESPOND_WAIT;
      break;
    case SKIP_EXT_RESPOND_WAIT:
      if(kbdhit())
      {
        ch = getchar();
        if(ch == 'n' || ch == 'N')
        {
          test_state = EXT_TEST_BEGIN_S;
          Error = 0;
          printf(NewLine);
        }
        else if (ch == 'y' || ch == 'Y')
        {
          test_state = FUNC_TEST_BEGIN_S;
          printf(NewLine);
        }
        else
        {
          test_state = TEST_BEGIN_S;
          printf(NewLine);
        }
      }
      break;
    case EXT_TEST_BEGIN_S:
      printf(NewLine);
      printf(ExtTest);
      for(i = 0; i < PIN_COUNT; ++i)
      {
        if(PinsSet[i])
        {
          SetDir(i,INPUT_DIR);
        }
      }
      // Pass 1 - Tets common Pull pin
      SetDir(PULL_UP_EXT_PIN,1);
      SetOut(PULL_UP_EXT_PIN,1);
//      SetDir(PULL_UP_UXT_PIN,1);
//      SetOut(PULL_UP_UXT_PIN,1);
      SetDir(PULL_UP_EMI_PIN,1);
      SetOut(PULL_UP_EMI_PIN,1);

      Dly100us((void *)10);
      for(i = 0; i < PIN_COUNT; ++i)
      {
        if(PinsSet[i] && (GetIn(i) == 0))
        {
          // Error
          Error = 1;
          // Show pin number
          printf(CommPullErr,PinNumb[PinsSet[i]],PinNumb[i]);
        }
      }

      if(Error)
      {
        printf(ExtTestFault);
        test_state = TEST_BEGIN_S;
      }
      else
      {
        test_state = EXT_TEST_PASS_2_S;
      }
      break;

    case EXT_TEST_PASS_2_S:
      // Pass 2 - test short between pins
      for(i = 0; i < PIN_COUNT; ++i)
      {
        if(PinsSet[i])
        {
          // Make a pin output
          SetDir(i,1);
          // Set oposite level from common test pin
          SetOut(i,0);
          Dly100us((void *)10);
          for (j = i+1; j < PIN_COUNT; ++j)
          {
            if((PinsSet[j] != 0)  && (GetIn(j) == 0))
            {
              // Show pin number
              printf(ShortErr,PinNumb[i],PinNumb[j]);
              Error = 1;
            }
          }
          // Make a pin input
          SetDir(i,0);
        }
      }

      if(Error)
      {
        printf(ExtTestFault);
        test_state = TEST_BEGIN_S;
      }
      else
      {
        test_state = EXT_TEST_PASS_3_S;
      }
      break;
    case EXT_TEST_PASS_3_S:
      SetOut(PULL_UP_EXT_PIN,0);
      //SetOut(PULL_UP_UXT_PIN,0);
      SetOut(PULL_UP_EMI_PIN,0);

      Dly100us((void *)10);
      for(i = 0; i < PIN_COUNT; ++i)
      {
        if(PinsSet[i] && (GetIn(i) == 1))
        {
          // Error
          Error = 1;
          // Show pin number
          printf(CommPullErr,PinNumb[PinsSet[i]],PinNumb[i]);
        }
      }

      if(Error)
      {
        printf(ExtTestFault);
        test_state = TEST_BEGIN_S;
      }
      else
      {
        test_state = EXT_TEST_PASS_4_S;
      }
      break;
    case EXT_TEST_PASS_4_S:
      for(i = 0; i < PIN_COUNT; ++i)
      {
        if(PinsSet[i])
        {
          // Make a pin output
          SetDir(i,1);
          // Set oposite level from common test pin
          SetOut(i,1);
          Dly100us((void *)10);
          for (j = i+1; j < PIN_COUNT; ++j)
          {
            if((PinsSet[j] != 0)  && (GetIn(j) == 1))
            {
              // Show pin number
              printf(ShortErr,PinNumb[i],PinNumb[j]);
              Error = 1;
            }
          }
          // Make a pin input
          SetDir(i,0);
        }
      }

      if(Error)
      {
        printf(ExtTestFault);
        test_state = TEST_BEGIN_S;
      }
      else
      {
        printf("\n\r EXT and EMI test OK");
        test_state = CAN_TEST_S;
      }
      break;

    case CAN_TEST_S:

      Error = CanTest();
      if(Error)
      {
        printf(ExtTestFault);
        test_state = TEST_BEGIN_S;
      }
      else
      {
        printf("\n\r CAN Test OK");
        test_state = ADC_TEST_S;
      }
      break;

    case ADC_TEST_S:

      Error = AdcTest();
      if(Error)
      {
        printf(AdcTestFault,Error-1);
        printf(ExtTestFault);
        test_state = TEST_BEGIN_S;
      }
      else
      {
        printf("\n\r ADC test OK");
        test_state = UEXT_TEST_S;
      }
      break;

    case UEXT_TEST_S:

      printf(UextTestMsg);
      ch = getchar();

      Error = UextTest();

      if(Error)
      {
        printf(ExtTestFault);
        test_state = TEST_BEGIN_S;
      }
      else
      {
        printf("\n\rUext Test OK");
        test_state = FUNC_TEST_BEGIN_S;
      }
    break;

    case FUNC_TEST_BEGIN_S:
      printf(NewLine);
      printf(FuncTest);
      printf(LanTest);
      about_fl = 0;
      test_state = FUNC_LAN_TEST_S;
      break;
    case FUNC_LAN_TEST_S:
      if(about_fl)
      {
        printf(TestPass);
        printf(USBTest);
        test_state = FUNC_USB_TEST_S;
        AudioClassInit(VIC_Slot2);
      }
      if(kbdhit())
      {
        ch = getchar();
        test_state = FUNC_USB_TEST_S;
        printf(TestSkipped);
        printf(USBTest);
        AudioClassInit(VIC_Slot2);
      }
      break;
    case FUNC_USB_TEST_S:
      if(kbdhit())
      {
        ch = getchar();
        test_state = FUNC_RTC_TEST_S;
        printf(TestSkipped);
      }
      if((UsbCoreReq(UsbCoreReqDevState) == UsbDevStatusConfigured) &&
         !UsbCoreReq(UsbCoreReqDevSusState))
      {
        printf(TestPass);
        test_state = FUNC_AUDIO_TEST_S;
      }
      break;
    case FUNC_AUDIO_TEST_S:
      printf(AudioTest);
      test_state = FUNC_AUDIO_TEST_RESPOND_S;
      break;
    case FUNC_AUDIO_TEST_RESPOND_S:
      if(kbdhit())
      {
        ch = getchar();
        if(ch == 'n' || ch == 'N')
        {
          test_state = FUNC_RTC_TEST_S;
          printf(FuncTestFault);
          Error = 0;
        }
        else if (ch == 'y' || ch == 'Y')
        {
          test_state = FUNC_RTC_TEST_S;
          printf(NewLine);
        }
        else
        {
          test_state = FUNC_AUDIO_TEST_S;
          printf(NewLine);
        }
      }
      break;
    case FUNC_RTC_TEST_S:
      printf(RTC_test);
      SCU_APBPeriphClockConfig(__RTC,ENABLE); /*enable RTC clock*/
      RTC_DeInit();
      RTC_TamperCmd(DISABLE);
      j = RTC_MILR;
      i = 10000;
      while (--i); /* do polling or RTC alarm flag*/
      if(j != RTC_MILR)
      {
        printf(TestPass);
      }
      else
      {
        printf(TestFault);
      }
      printf(TestMMC);
      // Init MMC
      MmcDiskInit();
      ProtStateCntr = 0;
      test_state = FUNC_MMC_TEST_S;
      break;
    case FUNC_MMC_TEST_S:
      // Update MMC Status
      MmcStatusUpdate();
      pMMCDiskStatus = MmcDiskStatus();

      if(kbdhit())
      {
        ch = getchar();
        test_state = FUNC_IR_TEST_S;
        printf(TestSkipped);
      }

      switch (pMMCDiskStatus->DiskStatus)
      {
      case DiskCommandPass:
        switch(pMMCDiskStatus->DiskType)
        {
        case DiskMMC:
          strcpy((char*)Message,"MMC Card - ");
          break;
        case DiskSD:
          strcpy((char*)Message,"SD Card - ");
          break;
        default:
          strcpy((char*)Message,"Card - ");
        }
        // Calculate MMC/SD size [MB]
        Tmp  = pMMCDiskStatus->BlockNumb * pMMCDiskStatus->BlockSize;
        Tmp  = Tmp/1000000;
        Tmp1 = Tmp/1000;
        if(Tmp1)
        {
          Message[strlen((char*)Message)+1] = 0;
          Message[strlen((char*)Message)]   = HexToCharStr[Tmp1];
          Tmp %= 1000;
        }
        Tmp1 = Tmp/100;
        if(Tmp1)
        {
          Message[strlen((char*)Message)+1] = 0;
          Message[strlen((char*)Message)]   = HexToCharStr[Tmp1];
          Tmp %= 100;
        }
        Tmp1 = Tmp/10;
        if(Tmp1)
        {
          Message[strlen((char*)Message)+1] = 0;
          Message[strlen((char*)Message)]   = HexToCharStr[Tmp1];
          Tmp %= 10;
        }
        if(Tmp)
        {
          Message[strlen((char*)Message)+1] = 0;
          Message[strlen((char*)Message)]   = HexToCharStr[Tmp];
        }
        strcat((char*)Message,"MB");
        for(Int32U i = strlen((char*)Message); i < 16; ++i)
        {
          Message[i] = ' ';
        }
        Message[strlen((char*)Message)+1] = 0;

        if(WriteProtect == pMMCDiskStatus->WriteProtect)
        {
          if(ProtStateCntr++ > 100)
          {
            test_state = FUNC_MMC_PRESEN_TEST_S;
            ProtStateCntr = 0;
            printf(TestPass);
            printf(TestWP_MMC);
          }
        }
        else
        {
          ProtStateCntr = 0;
          WriteProtect = pMMCDiskStatus->WriteProtect;
        }
        break;
      case DiskNotPresent:
      case DiskNotReady:
        strcpy((char*)Message,"Pls, Insert Card");
        break;
      default:
        printf(FuncTestFault);
        test_state = FUNC_IR_TEST_S;
      }
      //*// HD44780_StrShow(1, 2, (pInt8S)Message);
      // printf("\n\r");
      // printf((char const*)Message);
      break;
    case FUNC_MMC_PRESEN_TEST_S:
      // Update MMC Status
      MmcStatusUpdate();
      if(kbdhit())
      {
        ch = getchar();
        test_state = FUNC_IR_TEST_S;
        printf(TestSkipped);
      }
      pMMCDiskStatus = MmcDiskStatus();
      if (pMMCDiskStatus->DiskStatus == DiskNotPresent)
      {
        test_state = FUNC_MMC_PROT_TEST_S;
      }
      break;
    case FUNC_MMC_PROT_TEST_S:
      // Update MMC Status
      MmcStatusUpdate();
      if(kbdhit())
      {
        ch = getchar();
        test_state = FUNC_IR_TEST_S;
        printf(TestSkipped);
      }
      if (pMMCDiskStatus->DiskStatus == DiskCommandPass)
      {
        if(WriteProtectTmp == pMMCDiskStatus->WriteProtect)
        {
          if(++ProtStateCntr > 100)
          {
            if(WriteProtect != pMMCDiskStatus->WriteProtect)
            {
              printf(TestPass);

              #ifdef SPI_SEL_1
                printf(MMCAgain);
                getchar();
                #undef SPI_SEL_1
                break;
              #endif

              test_state = FUNC_IR_TEST_S;
            }
            else
            {
              if(WriteProtect)
              {
                printf(StallProtected);
              }
              else
              {
                printf(StallUnProtected);
              }
              printf(FuncTestFault);
              test_state = FUNC_IR_TEST_S;
            }
          }
        }
        else
        {
          ProtStateCntr = 0;
          WriteProtectTmp = pMMCDiskStatus->WriteProtect;
        }
      }
      break;
    case FUNC_IR_TEST_S:
      test_state = BATTERY_TEST_INIT_S;
      //sla test_state = FUNC_IR_TEST_BEGIN_S;
      //sla printf(TestIR);
      break;
    case FUNC_IR_TEST_BEGIN_S:
      if(kbdhit())
      {
        ch = getchar();
        if(ch == ' ')
        {
          printf(TestSkipped);
          test_state = BATTERY_TEST_INIT_S;
        }
        else if (ch == 'y' || ch == 'Y')
        {
          test_state = FUNC_IR_TEST_CONT_S;
          // init
          /*Gonfigure UART1_Rx pin GPIO3.2*/
          GPIO_InitStructure.GPIO_Direction = GPIO_PinInput;
          GPIO_InitStructure.GPIO_Pin = GPIO_Pin_2;
          GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull ;
          GPIO_InitStructure.GPIO_Alternate = GPIO_InputAlt1  ;
          GPIO_InitStructure.GPIO_IPConnected = GPIO_IPConnected_Enable;
          GPIO_Init (GPIO3, &GPIO_InitStructure);

          /*Gonfigure UART1_Tx pin GPIO3.3*/
          GPIO_InitStructure.GPIO_Direction = GPIO_PinOutput;
          GPIO_InitStructure.GPIO_Pin = GPIO_Pin_3;
          GPIO_InitStructure.GPIO_Type = GPIO_Type_PushPull ;
          GPIO_InitStructure.GPIO_IPConnected = GPIO_IPConnected_Enable;
          GPIO_InitStructure.GPIO_Alternate = GPIO_OutputAlt2  ;
          GPIO_Init (GPIO3, &GPIO_InitStructure);
          UART_Cmd(UART1, DISABLE);
          UART_StructInit(&UART_InitStructure);
          UART_InitStructure.UART_WordLength = UART_WordLength_8D;
          UART_InitStructure.UART_StopBits = UART_StopBits_1;
          UART_InitStructure.UART_Parity = UART_Parity_No ;
          UART_InitStructure.UART_BaudRate = 9600;
          UART_InitStructure.UART_HardwareFlowControl = UART_HardwareFlowControl_None;
          UART_InitStructure.UART_Mode = UART_Mode_Tx_Rx;
          UART_InitStructure.UART_FIFO = UART_FIFO_Disable;
          UART_Init(UART1, &UART_InitStructure);
          UART_IrDACmd(IrDA1,ENABLE);
          SCU_UARTIrDASelect(UART1,SCU_UARTMode_IrDA);
          UART_IrDASetCounter(IrDA1,18000000);
          UART_IrDACmd(IrDA1,ENABLE);
          UART_Cmd(UART1, ENABLE);
          Tmp = 100;
        }
        else
        {
          test_state = FUNC_IR_TEST_S;
          printf(NewLine);
        }
      }
      break;
    case FUNC_IR_TEST_CONT_S:
      //while(1)
      for(a = 100; a; --a)
      {
        Uart1SendChar(0x55);

        for(unsigned volatile int b = 50000; b; b--);

        if(Uart1ReceiveCharTest())
        {
          if(Uart1ReceiveChar() == 0x55)
          {
            printf(TestPass);
            test_state = BATTERY_TEST_INIT_S;
            break;
          }
        }
      }
      if(a==0)
      {
        printf(TestFault);
        test_state = BATTERY_TEST_INIT_S;
      }

      break;

    case BATTERY_TEST_INIT_S:
      RTC_DeInit();
      RTC_TamperConfig(RTC_TamperMode_Level,RTC_TamperPol_Low);
      RTC_TamperCmd(ENABLE);
      test_state = BATTERY_TEST_S;
    break;

    case BATTERY_TEST_S:

      if(!RTC_SR_bit.TSF)
      {
        printf("\n\rBattery test. Please put it and press any key to continue!");
        ch = getchar();
        if(RTC_SR_bit.TSF)
        {
          printf("\n\rBattery test OK!");
          test_state = FUNC_JOYSTICK_INIT_S;
        }
      }
      else
      {
        printf("\n\rBattery test fault!!!!!");
        test_state = FUNC_JOYSTICK_INIT_S;
      }


    break;

    case FUNC_JOYSTICK_INIT_S:
      SetDir(LCD_LIGHT,1);
      SetOut(LCD_LIGHT,1);
      printf(JoystickTest);
      test_state = FUNC_JOYSTICK_TEST_S;
      flag = 10;
      break;
    case FUNC_JOYSTICK_TEST_S:

      SetDir(BUT_CENTER,0);

      if((GetIn(BUT_CENTER) == 1)&&(flag!=0)) {
        printf("\rJoystick Press Center        ");
        SetOut(LCD_LIGHT,0);
        flag = 0;
      }

      Tmp = TestJoystick();

      if(((Tmp > 230) && (Tmp < 270))&&(flag!=1)) {
        printf("\rJoystick Press Right          ");
        SetOut(LCD_LIGHT,1);
        flag = 1;
      }
      else if(((Tmp > 40) && (Tmp < 80))&&(flag!=2)) {
        printf("\rJoystick Press Left         ");
        SetOut(LCD_LIGHT,1);
        flag = 2;
      }
      else if(((Tmp > 490) && (Tmp < 530))&&(flag!=3)) {
        printf("\rJoystick Press Up            ");
        SetOut(LCD_LIGHT,1);
        flag = 3;
      }
      else if(((Tmp > 110) && (Tmp < 150))&&(flag!=4)) {
        printf("\rJoystick Press Down          ");
        SetOut(LCD_LIGHT,1);
        flag = 4;
      }


      Tmp = TestTrim();
      if(((Tmp < 410) || (Tmp > 614))&&(trim_flag!=5))
      {
        printf("\rTrim Error                   ");
        trim_flag = 5;
      }


      if(((Tmp > 410) && (Tmp < 614))&&(trim_flag!=6)) {
        printf("\rTrim Ok                   ");
        trim_flag = 6;
      }

      break;

    default:
      test_state = EXT_TEST_BEGIN_S;
    }



    // TCP/IP
    // Let the tapdev network device driver read an entire IP packet
    //   into the uip_buf. If it must wait for more than 0.5 seconds, it
    //   will return with the return value 0. If so, we know that it is
    //   time to call upon the uip_periodic(). Otherwise, the tapdev has
    //   received an IP packet that is to be processed by uIP.
    uip_len = tapdev_read(uip_buf);

    if(uip_len)
    {
      if(BUF->type == htons(UIP_ETHTYPE_IP)) {
        uip_arp_ipin();
        uip_input();
        // If the above function invocation resulted in data that
        // should be sent out on the network, the global variable
        // uip_len is set to a value > 0.
        if(uip_len > 0)
        {
          uip_arp_out();
          TransmitPacket();
        }
      }
      else if(BUF->type == htons(UIP_ETHTYPE_ARP))
      {
        uip_arp_arpin();
        // If the above function invocation resulted in data that
        // should be sent out on the network, the global variable
        // uip_len is set to a value > 0.
        if(uip_len > 0)
        {
          TransmitPacket();
        }
      }
    }
    else if(ticks >= TICK_PER_SEC)
    {
      ticks = 0;
      for(i = 0; i < UIP_CONNS; i++)
      {
        uip_periodic(i);
        //  If the above function invocation resulted in data that
        // should be sent out on the network, the global variable
        // uip_len is set to a value > 0.
        if(uip_len > 0) {
          uip_arp_out();
          TransmitPacket();
        }
      }
#if UIP_UDP
      for(i = 0; i < UIP_UDP_CONNS; i++)
      {
        uip_udp_periodic(i);
        // If the above function invocation resulted in data that
        // should be sent out on the network, the global variable
        // uip_len is set to a value > 0.
        if(uip_len > 0)
        {
          uip_arp_out();
          TransmitPacket();
        }
      }
#endif // UIP_UDP

      // Call the ARP timer function every 10 seconds.
      if(++arptimer == 1)
      {
        uip_arp_timer();
        arptimer = 0;
      }

    }

  }
}
